import 'package:insurance_card_scanner/models/card_scan_state.dart';

/// Model of an HTTP response for getting card details
class CardResponse {
  /// Default constructor
  const CardResponse({
    required this.id,
    required this.state,
    required this.createdAt,
    required this.details,
    this.message,
  });

  /// Constructor from JSON
  factory CardResponse.fromJson(Map json) {
    // json properties are explicitly converted to string for preventing future
    // typing issues
    return CardResponse(
      id: json['cardId'],
      state: CardScanState.values.firstWhere(
        // doesn't hurt to lowercase unknown value
        (s) => '${json["state"]}'.toLowerCase() == s.name,
        orElse: () => CardScanState.error,
      ),
      createdAt: DateTime.tryParse('${json["createdAt"]}'),
      message: json['message'],
      details: json['details'] is Map<String, dynamic>
          ? CardDetails.fromJson(json['details'])
          : null,
    );
  }

  /// UUID of the card.
  final String id;

  /// Scanning state.
  final CardScanState state;

  /// Date when the scanning process was done.
  final DateTime? createdAt;

  /// Fields captured from the card.
  ///
  /// See more here: https://docs.cardscan.ai/api
  final CardDetails? details;

  /// Message associated to scanning process.
  ///
  /// This is generally null unless `state` is a [CardScanState.error]
  final String? message;
}

class CardDetails {
  const CardDetails({
    this.groupNumber,
    this.memberNumber,
    this.payerName,
    this.rxBin,
    this.rxPcn,
    this.rxGroup,
    this.memberName,
    this.dependentNames,
    this.planName,
    this.planId,
    this.cardSpecificId,
    this.startDate,
  });

  factory CardDetails.fromJson(Map<String, dynamic> json) => CardDetails(
        groupNumber: CardDetail.fromJson(json['groupNumber']),
        memberNumber: CardDetail.fromJson(json['memberNumber']),
        payerName: CardDetail.fromJson(json['payerName']),
        rxBin: CardDetail.fromJson(json['rxBin']),
        rxPcn: CardDetail.fromJson(json['rxPCN']),
        rxGroup: CardDetail.fromJson(json['rxGroup']),
        memberName: CardDetail.fromJson(json['memberName']),
        dependentNames: json['dependentNames'] is List
            ? (json['dependentNames'] as List)
                .map((e) => CardDetail.fromJson(e))
                .whereType<CardDetail>()
                .toList()
            : [],
        planName: CardDetail.fromJson(json['planName']),
        planId: CardDetail.fromJson(json['planId']),
        cardSpecificId: CardDetail.fromJson(json['cardSpecificId']),
        startDate: CardDetail.fromJson(json['startDate']),
      );

  final CardDetail? groupNumber;
  final CardDetail? memberNumber;
  final CardDetail? payerName;
  final CardDetail? rxBin;
  final CardDetail? rxPcn;
  final CardDetail? rxGroup;
  final CardDetail? memberName;
  final List<CardDetail>? dependentNames;
  final CardDetail? planName;
  final CardDetail? planId;
  final CardDetail? cardSpecificId;
  final CardDetail? startDate;
}

class CardDetail {
  const CardDetail(this.value, this.scores);

  /// Value for the detail
  final String value;

  // data capture accuracy
  final List<num> scores;

  static CardDetail? fromJson(Object? json) {
    if (json is Map) {
      try {
        return CardDetail(
            json['value'], (json['scores'] as List).cast<double>());
      } catch (e) {
        return null;
      }
    }
    return null;
  }
}
