import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:insurance_card_scanner/models/models.dart';
import 'package:insurance_card_scanner/utils/js_injection.dart';
import 'package:insurance_card_scanner/utils/logger.dart';
import 'package:pubspec_parse/pubspec_parse.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:webview_flutter_android/webview_flutter_android.dart';
import 'package:webview_flutter_wkwebview/webview_flutter_wkwebview.dart';

const _liveServer = 'api.cardscan.ai';

const _sandboxServer = String.fromEnvironment(
  'CARDSCAN_AI_SERVER',
  defaultValue: 'sandbox.cardscan.ai',
);

class ScannerWebView extends StatefulWidget {
  const ScannerWebView({super.key, required this.properties});

  final CardScanConfig properties;

  @override
  State<ScannerWebView> createState() => _ScannerWebViewState();
}

class _ScannerWebViewState extends State<ScannerWebView> {
  late final WebViewController _controller;

  String get webViewUrl =>
      widget.properties.live ? _liveServer : _sandboxServer;

  /// Retrieves version for the insurance_card_scanner package
  Future<String> getPackageVersion() async {
    final fileContent = await rootBundle.loadString(
      "packages/insurance_card_scanner/pubspec.yaml",
    );
    final pubspec = Pubspec.parse(fileContent);
    return pubspec.version!.canonicalizedVersion;
  }

  /// Loads content for the WebView
  Future<void> loadCardScannerUri(String packageVersion) {
    final osName = Platform.operatingSystem.toLowerCase();
    var version = Uri.encodeComponent(Platform.operatingSystemVersion);
    final uri = Uri(
      scheme: 'https',
      host: webViewUrl,
      path: 'v1/webview',
      queryParameters: {
        'platform': 'flutter',
        'os': osName,
        'version': version,
        'package': packageVersion
      },
    );

    return _controller.loadRequest(uri);
  }

  Future<void> injectJavaScript() =>
      _controller.runJavaScript(jsSnippet(widget.properties)).catchError((e) {
        logger(
          'Error: Failed to init CardScanView',
          error: e,
        );
      });

  void processMessage(String payload) {
    try {
      final message = jsonDecode(payload);
      if (message is Map) {
        switch (message['command']) {
          case 'cancel':
            logger('cardScanCancel');
            widget.properties.onCardScanCancel?.call();
            break;
          case 'success':
            logger('cardScanSuccess');
            final cardJson = message['card'];
            final card = cardJson is Map
                ? CardResponse.fromJson(cardJson).toCard()
                : null;
            if (card != null) {
              widget.properties.onSuccess?.call(card);
            } else {
              widget.properties.onCardScanError?.call(null);
            }
            break;
          case 'error':
            final error = message['error'];
            final errorMessage = error is Map ? error['message'] : null;
            logger("cardScanError - $error");
            widget.properties.onCardScanError?.call(errorMessage);
            break;
          case 'retry':
            logger('cardScanRetry');
            widget.properties.onCardScanRetry?.call();
            break;
          case 'viewLoaded':
            logger('CardScanViewLoaded');
            injectJavaScript();
            break;
          case 'syserror':
          case 'console.log':
          case 'console.error':
          case 'console.debug':
            var error = message['message'];
            logger("WebView - $error");
            break;
          default:
            logger("Unknown command received from JS - $message");
        }
      }
    } catch (e, stack) {
      logger(
        "Failed to parse JavascriptMessage - $payload",
        error: e,
        stackTrace: stack,
      );
    }
  }

  @override
  void initState() {
    super.initState();

    final params = WebViewPlatform.instance is WebKitWebViewPlatform
        ? WebKitWebViewControllerCreationParams(
            allowsInlineMediaPlayback: true,
            mediaTypesRequiringUserAction: const <PlaybackMediaTypes>{},
          )
        : const PlatformWebViewControllerCreationParams();

    final WebViewController controller =
        WebViewController.fromPlatformCreationParams(
      params,
      // camera permission is requested before rendering this widget
      onPermissionRequest: (request) => request.grant(),
    )
          ..setJavaScriptMode(JavaScriptMode.unrestricted)
          ..setOnConsoleMessage((message) => logger('JS: ${message.message}'))
          ..addJavaScriptChannel(
            'FlutterWebView',
            onMessageReceived: (message) {
              logger('onMessageReceived - messageHandler: ${message.message}');
              processMessage(message.message);
            },
          )
          ..setNavigationDelegate(
            NavigationDelegate(
              onPageFinished: (url) {
                logger('onPageFinished: $url');
              },
              onWebResourceError: (error) {
                logger(
                  "onReceivedError: ${error.errorCode} - ${error.description}",
                );

                widget.properties.onWebViewLoadError?.call();
              },
              onNavigationRequest: (request) {
                final uri = request.url;

                // prevent users' navigation inside the webview,
                // and return feedback on this
                if (!uri.contains('cardscan.ai') && !uri.contains('nop.sh')) {
                  widget.properties.onCardScanError?.call(
                    'Navigation to ${request.url} is not available',
                  );
                  return NavigationDecision.prevent;
                }

                return NavigationDecision.navigate;
              },
            ),
          )
          ..enableZoom(false);

    // debugging is not available for iOS devices
    if (controller.platform is AndroidWebViewController) {
      AndroidWebViewController.enableDebugging(widget.properties.debug);
      (controller.platform as AndroidWebViewController)
          .setMediaPlaybackRequiresUserGesture(false);
    }

    _controller = controller;
  }

  // @override
  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: getPackageVersion().then(loadCardScannerUri),
      builder: (context, snapshot) {
        if (snapshot.connectionState != ConnectionState.done) {
          return Container(); // your widget while loading
        }

        if (snapshot.hasError) {
          return Container(); //your widget when error happens
        }

        return WebViewWidget(controller: _controller); //place your widget here
      },
    );
  }
}
